@@ -1,12 +1,21 @@
 # Test::Reporter - sends test results to cpan-testers@perl.org
-# Copyright (c) 2007 Adam J. Foxson. All rights reserved.
-
+#
+# Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Adam J. Foxson.
+# Copyright (C) 2008 David A. Golden
+# Copyright (C) 2008 Ricardo Signes
+# Copyright (C) 2004, 2005 Richard Soderberg.
+# All rights reserved.
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 package Test::Reporter;
 
@@ -24,7 +33,7 @@ use constant FAKE_NO_NET_DNS => 0;    # for debugging only
 use constant FAKE_NO_NET_DOMAIN => 0; # for debugging only
 use constant FAKE_NO_MAIL_SEND => 0;  # for debugging only
 
-$VERSION = '1.38';
+$VERSION = '1.40';
 
 local $^W = 1;
 
@@ -42,7 +51,6 @@ sub new {
         '_comments'          => '',
         '_errstr'            => '',
         '_via'               => '',
-        '_mail_send_args'    => '',
         '_timeout'           => 120,
         '_debug'             => 0,
         '_dir'               => '',
@@ -54,6 +62,8 @@ sub new {
             '_myconfig' => Config::myconfig(),
         },
         '_transport'         => '',
+        '_transport_args'    => [],
+        '_mail_send_args'    => '', # deprecated -> use _transport_args
     };
 
     bless $self, $class;
@@ -107,14 +117,16 @@ sub _process_params {
 
     my %params   = @_;
     my @defaults = qw(
-        mx address grade distribution from comments via timeout debug dir perl_version transport);
+        mx address grade distribution from comments via timeout debug dir perl_version transport_args transport );
     my %defaults = map {$_ => 1} @defaults;
 
     for my $param (keys %params) {   
         croak __PACKAGE__, ": new: parameter '$param' is invalid." unless
             exists $defaults{$param};
     }
-
+    
+    # XXX need to process transport_args directly rather than through 
+    # the following -- store array ref directly
     for my $param (keys %params) {   
         $self->$param($params{$param});
     }
@@ -144,8 +156,6 @@ sub report {
     $report .= "This distribution has been tested as part of the cpan-testers\n";
     $report .= "effort to test as many new uploads to CPAN as possible.  See\n";
     $report .= "http://testers.cpan.org/\n\n";
-    $report .= "Please cc any replies to cpan-testers\@perl.org to keep other\n";
-    $report .= "test volunteers informed and to prevent any duplicate effort.\n";
 
     if (not $self->{_comments}) {
         $report .= "\n\n--\n\n";
@@ -185,23 +195,23 @@ sub transport {
     my $self = shift;
     warn __PACKAGE__, ": transport\n" if $self->debug();
 
-    my %transports    = (
-        # support for plugin transports will eventually be added, but not today
-        'Net::SMTP' => 'Builtin transport using Net::SMTP',
-        'Mail::Send' => 'Builtin transport using Mail::Send',
-    );
-
     return $self->{_transport} unless scalar @_;
 
     my $transport = shift;
 
-    croak __PACKAGE__, ":transport: '$transport' is invalid, choose from: " .
-        join ' ', keys %transports unless $transports{$transport};
+    my $transport_class = "Test::Reporter::Transport::$transport";
+    unless ( eval "require $transport_class; 1" ) { 
+        croak __PACKAGE__ . ": could not load '$transport_class'\n$@\n";
+    }
 
-    my $args = shift;
+    my @args = @_;
 
-    if ($transport eq 'Mail::Send' && defined $args && ref $args eq 'ARRAY') {
-        $self->mail_send_args($args);
+    if ( @args && $transport eq 'Mail::Send' && ref $args[0] eq 'ARRAY' ) {
+        # treat as old form of Mail::Send arguments and convert to list
+        $self->transport_args(@{$args[0]});
+    }
+    elsif ( @args ) {
+        $self->transport_args(@args);
     }
 
     return $self->{_transport} = $transport;
@@ -259,23 +269,16 @@ sub send {
         return;
     }
 
-    my $transport = $self->transport();
+    my $transport_type  = $self->transport() || 'Net::SMTP';
+    my $transport_class = "Test::Reporter::Transport::$transport_type";
+    my $transport = $transport_class->new( $self->transport_args() );
 
-    if ($transport eq 'Mail::Send' && $self->_have_mail_send()) {
-        return $self->_mail_send(@recipients);
-    }
-    elsif ($transport eq 'Net::SMTP') {
-        return $self->_send_smtp(@recipients);
-    }
-    else {
-        # Addresses #9831: Usage of Mail::Mailer is broken on Win32
-        if ($^O !~ /^(?:cygwin|MSWin32|VMS)$/ && $self->_have_mail_send()) {
-            return $self->_mail_send(@recipients);
-        }
-        else {
-            return $self->_send_smtp(@recipients);
-        }
+    unless ( eval { $transport->send( $self, \@recipients ) } ) {
+        $self->errstr(__PACKAGE__ . ": error from '$transport_class:'\n$@\n");
+        return;
     }
+
+    return 1;
 }
 
 sub write {
@@ -369,133 +372,6 @@ sub _verify {
     return $self->errstr() ? return 0 : return 1;
 }
 
-sub _mail_send {
-    my $self = shift;
-    warn __PACKAGE__, ": _mail_send\n" if $self->debug();
-
-    my $fh;
-    my $recipients;
-    my @recipients = @_;
-    my $via        = $self->via();
-    my $msg        = Mail::Send->new();
-
-    if (@recipients) {
-        $recipients = join ', ', @recipients;
-        chomp $recipients;
-        chomp $recipients;
-    }
-
-    $via = ', via ' . $via if $via;
-
-    $msg->to($self->address());
-    $msg->set('From', $self->from());
-    $msg->subject($self->subject());
-    $msg->add('X-Reported-Via', "Test::Reporter ${VERSION}$via");
-    $msg->add('Cc', $recipients) if @_;
-
-    if ($self->mail_send_args() and ref $self->mail_send_args() eq 'ARRAY') {
-        $fh = $msg->open(@{$self->mail_send_args()});
-    }
-    else {
-        $fh = $msg->open();
-    }
-
-    print $fh $self->report();
-    
-    $fh->close();
-}
-
-sub _send_smtp {
-    my $self = shift;
-    warn __PACKAGE__, ": _send_smtp\n" if $self->debug();
-
-    my $helo          = $self->_maildomain();
-    my $from          = $self->from();
-    my $via           = $self->via();
-    my $debug         = $self->debug();
-    my @recipients    = @_;
-    my @tmprecipients = ();
-    my @bad           = ();
-    my $success       = 0;
-    my $fail          = 0;
-    my $recipients;
-    my $smtp;
-
-    my $mx;
-    for my $server (@{$self->{_mx}}) {
-        $smtp = Net::SMTP->new($server, Hello => $helo,
-            Timeout => $self->{_timeout}, Debug => $debug);
-
-        if (defined $smtp) {
-            $mx = $server;
-            last;
-        }
-        else {
-            warn __PACKAGE__, ": Unable to connect to MX '$server'\n" if $self->debug();
-            $fail++;
-        }
-    }
-
-    unless ($mx && $smtp) {
-        $self->errstr(__PACKAGE__ . ': Unable to connect to any MX\'s');
-        return 0;
-    }
-
-    if (@recipients) {
-        if ($mx =~ /(?:^|\.)(?:perl|cpan)\.org$/) {
-            for my $recipient (sort @recipients) {
-                if ($recipient =~ /(?:@|\.)(?:perl|cpan)\.org$/) {
-                    push @tmprecipients, $recipient;
-                } else {
-                    push @bad, $recipient;
-                }
-            }
-
-            if (@bad) {
-                warn __PACKAGE__, ": Will not attempt to cc the following recipients since perl.org MX's will not relay for them. Either install Mail::Send, use other MX's, or only cc address ending in cpan.org or perl.org: ${\(join ', ', @bad)}.\n";
-            }
-
-            @recipients = @tmprecipients;
-        }
-
-        $recipients = join ', ', @recipients;
-        chomp $recipients;
-        chomp $recipients;
-    }
-
-    $via = ', via ' . $via if $via;
-
-    my $envelope_sender = $from;
-    $envelope_sender =~ s/\s\([^)]+\)$//; # email only; no name
-
-    $success += $smtp->mail($envelope_sender);
-    $success += $smtp->to($self->{_address});
-    $success += $smtp->cc(@recipients) if @recipients;
-    $success += $smtp->data();
-    $success += $smtp->datasend("Date: ", $self->_format_date, "\n");
-    $success += $smtp->datasend("Subject: ", $self->subject(), "\n");
-    $success += $smtp->datasend("From: $from\n");
-    $success += $smtp->datasend("To: ", $self->{_address}, "\n");
-    $success += $smtp->datasend("Cc: $recipients\n") if @recipients && $success == 8;
-    $success += $smtp->datasend("Message-ID: ", $self->message_id(), "\n");
-    $success +=
-        $smtp->datasend("X-Reported-Via: Test::Reporter ${VERSION}$via\n");
-    $success += $smtp->datasend("\n");
-    $success += $smtp->datasend($self->report());
-    $success += $smtp->dataend();
-    $success += $smtp->quit;
-
-    if (@recipients) {
-        $self->errstr(__PACKAGE__ .
-            ": Unable to send test report to one or more recipients\n") if $success != 15;
-    }
-    else {
-        $self->errstr(__PACKAGE__ . ": Unable to send test report\n") if $success != 13;
-    }
-
-    return $self->errstr() ? 0 : 1;
-}
-
 # Courtesy of Email::MessageID
 sub message_id {
     my $self = shift;
@@ -543,21 +419,36 @@ sub mx {
     return $self->{_mx};
 }
 
+# Deprecated, but kept for backwards compatibility
+# Passes through to transport_args -- converting from array ref to list to
+# store and converting from list to array ref to get
 sub mail_send_args {
     my $self = shift;
     warn __PACKAGE__, ": mail_send_args\n" if $self->debug();
     croak __PACKAGE__, ": mail_send_args cannot be called unless Mail::Send is installed\n" unless $self->_have_mail_send();
-
     if (@_) {
         my $mail_send_args = shift;
-        croak __PACKAGE__, ": mail_send_args: array reference required" if
-            ref $mail_send_args ne 'ARRAY';
-        $self->{_mail_send_args} = $mail_send_args;
+        croak __PACKAGE__, ": mail_send_args: array reference required\n" 
+            if ref $mail_send_args ne 'ARRAY';
+        $self->transport_args(@$mail_send_args);
+    }
+    return [ $self->transport_args() ];
+}
+
+
+
+sub transport_args {
+    my $self = shift;
+    warn __PACKAGE__, ": transport_args\n" if $self->debug();
+    
+    if (@_) {
+        $self->{_transport_args} = ref $_[0] eq 'ARRAY' ? $_[0] : [ @_ ];
     }
 
-    return $self->{_mail_send_args};
+    return @{ $self->{_transport_args} };
 }
 
+
 sub perl_version  {
     my $self = shift;
     warn __PACKAGE__, ": perl_version\n" if $self->debug();
@@ -796,43 +687,7 @@ sub _is_a_perl_release {
     return $perl =~ /^perl-?\d\.\d/;
 }
 
-
-# Next two subs courtesy of Casey West, Ricardo SIGNES, and Email::Date
-# Visit the Perl Email Project at: http://emailproject.perl.org/
-sub _tz_diff {
-    my $self = shift;
-    warn __PACKAGE__, ": _tz_diff\n" if $self->debug();
-
-    my ($time) = @_;
-
-    my $diff  =   Time::Local::timegm(localtime $time)
-                - Time::Local::timegm(gmtime    $time);
-
-    my $direc = $diff < 0 ? '-' : '+';
-       $diff  = abs $diff;
-    my $tz_hr = int( $diff / 3600 );
-    my $tz_mi = int( $diff / 60 - $tz_hr * 60 );
-
-    return ($direc, $tz_hr, $tz_mi);
-}
-
-sub _format_date {
-    my $self = shift;
-    warn __PACKAGE__, ": _format_date\n" if $self->debug();
-
-    my ($time) = @_;
-    $time = time unless defined $time;
-
-    my ($sec, $min, $hour, $mday, $mon, $year, $wday) = (localtime $time);
-    my $day   = (qw[Sun Mon Tue Wed Thu Fri Sat])[$wday];
-    my $month = (qw[Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec])[$mon];
-    $year += 1900;
-
-    my ($direc, $tz_hr, $tz_mi) = $self->_tz_diff($time);
-
-    sprintf "%s, %d %s %d %02d:%02d:%02d %s%02d%02d",
-      $day, $mday, $month, $year, $hour, $min, $sec, $direc, $tz_hr, $tz_mi;
-}
+__END__
 
 =head1 NAME
 
@@ -912,7 +767,7 @@ Discussion group for Test::Reporter
 
 The Wiki for Test::Reporter
 
-=item * L<http://repo.or.cz/w/test-reporter.git>
+=item * L<http://eclipse.resort.org/git/gitweb.cgi?p=test-reporter.git>
 
 Test::Reporter's public git source code repository.
 
@@ -1011,12 +866,14 @@ result. This must be one of:
   na        distribution will not work on this platform
   unknown   distribution did not include tests
 
-=item * B<mail_send_args>
+=item * B<mail_send_args> -- DEPRECATED
+
+Kept for backwards compatibility.  Use C<transport_args> instead.
 
 Optional. If you have MailTools installed and you want to have it
 behave in a non-default manner, parameters that you give this
 method will be passed directly to the constructor of
-Mail::Mailer. See L<Mail::Mailer> and L<Mail::Send> for details. 
+Mail::Mailer. See L<Mail::Mailer> and L<Mail::Send> for details.
 
 =item * B<message_id>
 
@@ -1079,23 +936,39 @@ reports. Default is 120 seconds.
 
 =item * B<transport>
 
-Optional. Gets or sets the transport method. If you do not specify a transport,
-one will be selected automatically on your behalf: If you're on Windows,
-Net::SMTP will be selected, if you're not on Windows, Net::SMTP will be
-selected unless Mail::Send is installed, in which case Mail::Send is used.
+Optional. Gets or sets the transport type. The transport type argument is 
+refers to a 'Test::Reporter::Transport' subclass.  The default is 'Net::SMTP',
+which uses the [Test::Reporter::Transport::Net::SMTP] class.
 
-At the moment, this must be one of either 'Net::SMTP', or 'Mail::Send'.
-Support for authenticated SMTP may soon be possibly added as well.
+You can add additional arguments after the transport
+selection.  These will be passed to the constructor of the lower-level
+transport. This can be used to great effect for all manner of fun and
+enjoyment. ;-) See C<transport_args>.
 
-If you specify 'Mail::Send' as a transport, you can add an additional
-argument in the form of an array reference which will be passed to the
-constructor of the lower-level Mail::Mailer. This can be used to great
-effect for all manner of fun and enjoyment. ;-)
+If L<Net::SMTP::TLS> is used, 'Username' and 'Password' key-value transport
+arguments must be provided.
+
+ $reporter->transport( 
+     'Net::SMTP::TLS', Username => 'jdoe', Password => '123' 
+ );
+
+If the 'HTTP' transport is used, two additional arguments are required: 
+a URL to a L<Test::Reporter::HTTPGateway> compatible server and an (optional)
+API key.
+
+ $reporter->transport( 
+     'HTTP', 'http://example.com/reporter-gateway/', '123456' 
+ );
 
 This is not designed to be an extensible platform upon which to build
 transport plugins. That functionality is planned for the next-generation
 release of Test::Reporter, which will reside in the CPAN::Testers namespace.
 
+=item * B<transport_args>
+
+Optional.  Gets or sets transport arguments that will used in the constructor
+for the selected transport, as appropriate.
+
 =item * B<via>
 
 Optional. Gets or sets the value that will be appended to
@@ -1142,9 +1015,16 @@ Mail::Send installed) be sure that you use the author's @cpan.org address
 otherwise they may not be delivered, since the perl.org MX's are unlikely
 to relay for anything other than perl.org and cpan.org.
 
+If you experience a long delay sending mail with Test::Reporter, you may be 
+experiencing a wait as Test::Reporter attempts to determine your email 
+domain.  Setting the MAILDOMAIN environment variable will avoid this delay.
+
 =head1 COPYRIGHT
 
-Copyright (c) 2007 Adam J. Foxson. All rights reserved.
+ Copyright (C) 2008 David A. Golden.
+ Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Adam J. Foxson.
+ Copyright (C) 2004, 2005 Richard Soderberg.
+ All rights reserved.
 
 =head1 LICENSE
 
@@ -1161,6 +1041,8 @@ and/or modify it under the same terms as Perl itself.
 
 =item * L<Net::SMTP>
 
+=item * L<Net::SMTP::TLS>
+
 =item * L<File::Spec>
 
 =item * L<File::Temp>
@@ -1181,6 +1063,10 @@ MX's known at the time of this release.
 This is optional. If it's installed Test::Reporter will use Mail::Send
 instead of Net::SMTP.
 
+=item * L<Test::Reporter::HTTPGateway>
+
+This is optional.  It provides a web API for the 'HTTP' transport method.
+
 =back
 
 =head1 AUTHOR
@@ -1191,6 +1077,8 @@ Kirrily "Skud" Robert E<lt>F<skud@cpan.org>E<gt>, and
 Kurt Starsinic E<lt>F<Kurt.Starsinic@isinet.com>E<gt> for predecessor versions
 (CPAN::Test::Reporter, and cpantest respectively).
 
+Additional contributions by David A. Golden and Ricardo Signes.
+
 =cut
 
 1;
