@@ -1,8 +1,8 @@
 use strict;
-BEGIN{ if (not $] < 5.006) { require warnings; warnings->import } }
+use warnings;
 package Test::Reporter::Transport;
 use vars qw/$VERSION/;
-$VERSION = '1.53_03';
+$VERSION = '1.40';
 $VERSION = eval $VERSION;
 
 sub new     { die "Not implemented" }
