@@ -1,9 +1,9 @@
 use strict;
-BEGIN{ if (not $] < 5.006) { require warnings; warnings->import } }
+use warnings;
 package Test::Reporter::Transport::File;
 use base 'Test::Reporter::Transport';
 use vars qw/$VERSION/;
-$VERSION = '1.53_03';
+$VERSION = '1.40';
 $VERSION = eval $VERSION;
 
 sub new {
@@ -44,7 +44,7 @@ the C<write> method from Test::Reporter.
 This lets you save reports during offline operation.  The files may later be
 uploaded using C<< Test::Reporter->read() >>.
 
-    Test::Reporter->new->read( $file )->send();
+    Test::Reporter->read( $file )->send();
 
 =head1 USAGE
 
