@@ -1,9 +1,9 @@
 use strict;
-BEGIN{ if (not $] < 5.006) { require warnings; warnings->import } }
+use warnings;
 package Test::Reporter::Transport::HTTPGateway;
 use base 'Test::Reporter::Transport';
 use vars qw/$VERSION/;
-$VERSION = '1.53_03';
+$VERSION = '1.40';
 $VERSION = eval $VERSION;
 
 use LWP::UserAgent;
